@@ -15,17 +15,22 @@ from django.contrib.auth import get_user_model
 from django.http import JsonResponse, HttpResponseRedirect
 from django.db import transaction
 from django.core.exceptions import PermissionDenied
+from django.db.models import Q, Sum, Count
+
 from qiniu import Auth
+from qiniu import BucketManager
+
 from django.conf import settings
 from django.views.decorators.csrf import csrf_exempt
 from django.contrib.auth.decorators import login_required, permission_required
+from dss.Serializer import serializer
+from django.db.models.signals import pre_delete
+from django.dispatch import receiver
 
-from .models import Course, Section, Part, Video, Question, Option, UserCourse, UserSection, UserPart, UserVideo, \
-    UserQuestion
-from .forms import CourseForm, SectionForm, PartForm, PartUpdateForm, VideoForm, QuestionForm, QuestionSearchForm, \
-    OptionForm, \
-    UploadImageForm, \
-    UserCourseForm
+from .models import Course, Section, Part, Video, Question, Option, UserCourse, UserSection, \
+    UserPart, UserVideo, UserQuestion
+from .forms import CourseForm, SectionForm, PartForm, PartUpdateForm, VideoForm, QuestionForm, \
+    QuestionSearchForm, OptionForm, UploadImageForm, UserCourseForm
 
 User = get_user_model()
 
@@ -137,7 +142,7 @@ class SectionUpdate(UpdateView):
 
 class SectionDelete(DeleteView):
     model = Section
-    success_url = reverse_lazy('bee_django_course_simple:section_list')
+    success_url = reverse_lazy('bee_django_course_simple:course_list')
 
     def get(self, request, *args, **kwargs):
         return self.http_method_not_allowed(request, *args, **kwargs)
@@ -219,7 +224,7 @@ class PartUpdate(UpdateView):
 
 class PartDelete(DeleteView):
     model = Part
-    success_url = reverse_lazy('bee_django_course_simple:section_list')
+    success_url = reverse_lazy('bee_django_course_simple:course_list')
 
     def get(self, request, *args, **kwargs):
         return self.http_method_not_allowed(request, *args, **kwargs)
@@ -328,6 +333,42 @@ def video_detail(request, video_id):
     })
 
 
+def delete_video(request, video_id):
+    if request.method == "POST":
+        video = get_object_or_404(Video, pk=video_id)
+
+        if request.user.has_perm('bee_django_course_simple.change_section'):
+            video.delete()
+            return JsonResponse(data={
+                'rc': 0,
+                'message': '删除成功'
+            })
+        else:
+            return JsonResponse(data={
+                'rc': -1,
+                'message': '权限不足'
+            })
+
+
+@receiver(pre_delete, sender=Video, dispatch_uid='video_delete_singal')
+def delete_qiniu_video(sender, instance, using, **kwargs):
+    key = instance.file_name
+
+    access_key = settings.QINIU_AK
+    secret_key = settings.QINIU_SK
+    # 构建鉴权对象
+    q = Auth(access_key, secret_key)
+    # 初始化BucketManager
+    bucket = BucketManager(q)
+
+    # 要上传的空间
+    bucket_name = settings.QINIU_BUCKET_NAME
+
+    # 删除bucket_name 中的文件 key
+    ret, info = bucket.delete(bucket_name, key.encode('utf-8'))
+    # print(info)
+
+
 # ========== 小节为问题==============
 class QuestionCreate(CreateView):
     model = Question
@@ -399,6 +440,53 @@ class QuestionDetail(DetailView):
     template_name = 'bee_django_course_simple/question/question_detail.html'
     context_object_name = 'question'
 
+class QuestionRecordDetail(DetailView):
+    model = Question
+    template_name = 'bee_django_course_simple/question/question_reocrd_detail.html'
+    context_object_name = 'question'
+
+    def get_context_data(self, **kwargs):
+        context = super(QuestionRecordDetail, self).get_context_data(**kwargs)
+        question_id = self.kwargs["pk"]
+        question = Question.objects.get(id=question_id)
+        option_list = Option.objects.filter(question=question)
+        res_queryset = UserQuestion.objects.filter(question=question).values('answer_option_id').annotate(
+            answer_count=Count('answer_option')).order_by("-answer_count")
+
+        res_list = []
+        user_count = 0
+        print(option_list)
+        print(res_queryset)
+        for option in option_list:
+            d = {}
+            found = False
+            print('===')
+            print(option)
+            for i in res_queryset:
+                answer_option_id = i["answer_option_id"]
+
+                print(answer_option_id)
+                if not answer_option_id:
+                    found = False
+                elif option.id == answer_option_id:
+                    d["key"] = option.title
+                    d["value"] = i["answer_count"]
+                    user_count += i["answer_count"]
+                    found = True
+                    res_list.append(d)
+            if found == False:
+                d["key"] = option.title
+                d["value"] = 0
+                res_list.append(d)
+
+        # for res_dict in res_queryset:
+        #     # option_id = res_dict["answer"]
+        #     count = res_dict["answer_count"]
+
+        context["user_count"] = user_count
+        context["res_list"] = res_list
+        # context["res_queryset"]=res_queryset
+        return context
 
 class QuestionUpdate(UpdateView):
     model = Question
@@ -418,7 +506,7 @@ class QuestionUpdate(UpdateView):
 
 class QuestionDelete(DeleteView):
     model = Question
-    success_url = reverse_lazy('bee_django_course_simple:part_list')
+    success_url = reverse_lazy('bee_django_course_simple:course_list')
 
     def get(self, request, *args, **kwargs):
         return self.http_method_not_allowed(request, *args, **kwargs)
@@ -468,7 +556,7 @@ class OptionUpdate(UpdateView):
 
 class OptionDelete(DeleteView):
     model = Option
-    success_url = reverse_lazy('bee_django_course_simple:part_list')
+    success_url = reverse_lazy('bee_django_course_simple:course_list')
 
     def get(self, request, *args, **kwargs):
         return self.http_method_not_allowed(request, *args, **kwargs)
@@ -560,7 +648,8 @@ class CustomUserSectionDetailRedirect(RedirectView):
         user_course_list = UserCourse.objects.filter(user__id=user_id).order_by("status")
         if user_course_list.exists():
             user_course = user_course_list.first()
-            user_section_list = user_course.usersection_set.all().order_by("status", "section__number")
+            #  只找正在学的和已通过的。
+            user_section_list = user_course.usersection_set.filter(status__gte=1).order_by("status", "section__number")
             if user_course_list.exists():
                 user_section = user_section_list.first()
                 self.url = reverse('bee_django_course_simple:custom_user_section_detail',
@@ -576,6 +665,13 @@ class CustomUserSectionDetailRedirect(RedirectView):
 class CustomUserSectionDetail(UserSectionDetail):
     template_name = 'bee_django_course_simple/section/custom_user_section_detail.html'
 
+    def get_context_data(self, **kwargs):
+        context = super(CustomUserSectionDetail, self).get_context_data(**kwargs)
+        context["qiniu_domain"] = settings.QINIU_DOMAIN
+        user_section = UserSection.objects.get(id=self.kwargs["pk"])
+        context["next_user_section"] = user_section.next_user_section(status=1)
+        return context
+
 
 class CustomUserSectionEmpty(TemplateView):
     template_name = 'bee_django_course_simple/section/custom_user_section_empty.html'
@@ -588,18 +684,153 @@ class UserSectionUpdate(TemplateView):
         user_section = UserSection.objects.get(id=user_section_id)
         msg = ''
         if type == 'open':
-            user_section.open_section()
+            user_section._open()
             msg = '开启'
         elif type == 'pass':
-            user_section.pass_section()
-            next_section = user_section.next_section(status=0)
-            if next_section:
-                next_section.open_section()
+            user_section._pass()
+            # user section下的所有user part也通过
+            for user_part in user_section.userpart_set.all():
+                user_part._pass()
+            next_user_section = user_section.next_user_section(status=0)
+            if next_user_section:
+                next_user_section._open()
             msg = '通过'
         elif type == 'close':
-            user_section.close_section()
+            user_section._close()
+            # user section下的所有user part也关闭
+            for user_part in user_section.userpart_set.all():
+                user_part._close()
             msg = '关闭'
         return JsonResponse(data={
             'rc': 0,
             'message': msg + '成功'
         })
+
+
+class UserPartUpdate(TemplateView):
+    def post(self, request, *args, **kwargs):
+        user_part_id = self.kwargs["user_part_id"]
+        question_data = self.request.POST.get("question_data")
+        video_id = self.request.POST.get("video_id")
+
+        next_user_part_id = None
+        next_user_section_id = None
+        next_user_section_url = None
+        user_part = UserPart.objects.get(id=user_part_id)
+        question_count = 0  # 小节中题目数量
+        question_correct_count = 0  # 小节中答对题目数量
+
+        # 保存记录,更新part内question或video状态
+        if question_data:
+            question_data = json.loads(question_data)
+            for question in question_data:
+                # 记录过则不记录
+                u_q = UserQuestion.objects.filter(user_part=user_part, question_id=question['question_id'])
+                if u_q.exists():
+                    continue
+                user_question = UserQuestion()
+                user_question.user_part = user_part
+                user_question.question_id = question["question_id"]
+                user_question.answer_option_id = question["option_id"]
+                user_question.save()
+
+                # 该小节问题的数量
+                question_count = Question.objects.filter(part=user_part.part).count()
+                # 检查答案
+                corrent_option = Option.objects.filter(question__id=question["question_id"], id=question["option_id"],
+                                                       is_correct=True)
+                if corrent_option:
+                    question_correct_count += 1
+
+        if video_id:
+            video = Video.objects.get(id=video_id)
+            # 记录过则不记录
+            u_v = UserVideo.objects.filter(user_part=user_part, video=video)
+            if not u_v.exists():
+                video = Video.objects.get(id=video_id)
+                user_video = UserVideo()
+                user_video.user_part = user_part
+                user_video.video = video
+                user_video.save()
+
+        # 判断当前user part是否可以通过，更新user part 状态
+        user_part_can_pass = user_part.can_pass()
+        if user_part_can_pass and user_part.is_open():
+            user_part._pass()
+
+        # 查找下一个user part，检查自动开启
+        next_user_part = user_part.next_user_part(status=0)
+        if next_user_part:
+            next_user_part_id = next_user_part.id
+            if user_part_can_pass and next_user_part.is_close():
+                next_user_part._open()
+
+        # 判断当前user section是否可以通过
+        user_section_can_pass = user_part.user_section.can_pass()
+        if user_section_can_pass and user_part.user_section.is_open():
+            user_part.user_section._pass()
+
+        # 查找下一个user section，检查自动开启
+        next_user_section = user_part.user_section.next_user_section(status=0)
+        if next_user_section and user_section_can_pass:
+            next_user_section._open()
+            next_user_section_id = next_user_section.id
+            next_user_section_url = reverse('bee_django_course_simple:custom_user_section_detail',
+                                            kwargs={'pk': next_user_section_id})
+
+        return JsonResponse(data={
+            'error': 0,
+            'message': '成功',
+            'next_user_part_id': next_user_part_id,
+            'next_user_section_id': next_user_section_id,
+            'next_user_section_url': next_user_section_url,
+            'user_part_can_pass': user_part_can_pass,
+            "user_section_can_pass": user_section_can_pass,
+            "question_count": question_count,
+            "question_correct_count": question_correct_count,
+        })
+
+
+# VueJS 拉取数据用
+class VueUserSection(TemplateView):
+    def get(self, request, *args, **kwargs):
+        user_section_id = self.kwargs['user_section_id']
+        user_section = get_object_or_404(UserSection, pk=user_section_id)
+        user_parts = user_section.userpart_set.all()
+
+        for user_part in user_parts:
+            user_part.part_type = user_part.part.type
+            user_part.part_title = user_part.part.title
+            user_part.extra_title = user_part.part.extra_title
+
+            if user_part.part_type == 1:
+                videos = []
+                for v in user_part.part.video_set.all():
+                    videos.append({
+                        'id': v.id,
+                        'content': v.content,
+                        'url': "http://" + settings.QINIU_DOMAIN + "/" + v.file_name
+                    })
+                user_part.videos = videos
+            elif user_part.part_type == 2:
+                questions = []
+                for q in user_part.part.question_set.all():
+                    options = []
+                    for o in q.option_set.all():
+                        options.append({
+                            'id': o.id,
+                            'title': o.title
+                        })
+                    questions.append({
+                        'id': q.id,
+                        'title': q.title,
+                        'options': options
+                    })
+                user_part.questions = questions
+
+        data = serializer(user_parts, output_type='json', datetime_format='string')
+
+        return JsonResponse(data={
+            'user_parts': data,
+            'status': user_section.status,
+        })
